#
# and-or.py, 11 Oct 24
#
# Convert subclauses in a conditional expression into single variables

import csv
from pycparser.c_lexer import CLexer
import sys


# The following CLexer code is adapted from:
# https://github.com/eliben/pycparser/blob/main/tests/test_c_lexer.py

def token_list(clex):
    return list(iter(clex.token, None))


def token_types(clex):
    return [i.type for i in token_list(clex)]

class C_lexer():
    # Lexing can cause errors.
    def error_func(self, msg, line, column):
        print("Error:", msg)

    def type_lookup_func(self, typ):
        if typ.startswith('mytype'):
            return True
        else:
            return False

    def setUp(self):
        self.clex = CLexer(self.error_func, lambda: None, lambda: None,
                           self.type_lookup_func)
        self.clex.build(optimize=False)

    def get_tokens(self, str):
        self.clex.input(str)
        return token_types(self.clex)


# Now my clause extraction code

def print_dummy():
   global pending_var
   global at_start
   if not at_start:
      print(" ", sep="", end="")
   at_start=False
   print("dm", sep="", end="")
   pending_var=3

def print_var():
   global num_clause
   global pending_var
   global at_start
   if pending_var == 3:
      return
   if pending_var == 2:
      print_dummy()
   else:
      if not at_start:
         print(" ", sep="", end="")
      at_start=False
      num_clause+=1
      print("v", num_clause, sep="", end="")
      pending_var=0

def print_tokens(tok):
   global at_start
   if not at_start:
      print(" ", sep="", end="")
   at_start=False
   print(tok, sep="", end="")

def extract_clauses(expr):
   global num_clause
   global pending_var
   global at_start

   at_start=True
   num_and=0
   num_or=0
   num_tern=0
   num_clause=0
   nest_paren=0
   last_paren=0
   nest_sqr=0
   last_sqr=0
   seen_something=0
   pending_var=1
   for token in expr:
      # print(token.token_value, ".", end="")
      # print(token)
      match token: # requires at least version 3.10
         case 'LPAREN':
                 nest_paren+=1
         case 'RPAREN':
                 # don't print for ) ), i.e., a && ( b || ( c && d ) )
                 if last_paren == nest_paren and seen_something:
                    print_var()
                 if last_paren == nest_paren:
                    print_tokens(")")
                    last_paren=nest_paren-1
                    seen_something=0
                 nest_paren-=1
         case 'LAND': # handle: a && ( ( b && c ) || d ) && e
                 for x in range(last_paren , nest_paren):
                    print_tokens("(")
                 if seen_something:
                    print_var()
                 print_tokens("&&")
                 num_and+=1
                 pending_var=1
                 last_paren=nest_paren
         case 'LOR':
                 for x in range(last_paren , nest_paren):
                    print_tokens("(")
                 if seen_something:
                    print_var()
                 print_tokens("||")
                 num_or+=1
                 pending_var=1
                 last_paren=nest_paren
         case 'CONDOP':
                 for x in range(last_paren , nest_paren):
                    print_tokens("(")
                 if seen_something:
                    pending_var=1 # handle a sequence of ternary operators
                    print_var()
                 print_tokens("?")
                 num_tern+=1
                 pending_var=2 # ideally we want a dummy variable
                 last_paren=nest_paren
         case 'COLON':
                 for x in range(last_paren , nest_paren):
                    print_tokens("(")
                 if seen_something:
                    print_dummy()
                 print_tokens(":")
                 pending_var=2 # ideally we want a dummy variable
                 last_paren=nest_paren
         case " ":
                 continue # ignore
         case _:
                 seen_something=1
   if pending_var or seen_something:
      print_var()
   # if nest_paren != 0:
   #    print("ERROR: paren nesting != 0 ==", nest_paren)
   print(",", num_and, ",", num_or, ",", num_tern, sep="")


num_clause=0
pending_var=0
at_start=True

tokenizer = C_lexer()
tokenizer.setUp()

# with open('bug.txt', newline='') as csvfile:
with open(sys.argv[1], newline='') as csvfile:
   file_info = csv.DictReader(csvfile, delimiter=';')
   print("simp_expr,land,lor,ternary")
   for row in file_info:
       # print(row['if_line'], row['expr'])
       if row['stmt'] == 'else' or row['stmt'] == 'func':
          print("NA,NA,NA,NA")
          continue
       tokens = tokenizer.get_tokens(row['expr'])
       # print(tokens)
       extract_clauses(tokens)

